@@ -1185,6 +1185,7 @@ primes_interval(GEN a, GEN b)
   GEN y, d, p;
   if (typ(a) != t_INT) a = gceil(a);
   if (typ(b) != t_INT) b = gfloor(b);
+  if (signe(a) < 0) a = gen_2;
   d = subii(b, a);
   if (signe(d) < 0 || signe(b) <= 0) { avma = av; return cgetg(1, t_VEC); }
   if (lgefint(b) == 3)
@@ -1230,7 +1231,11 @@ primes_interval_zv(ulong a, ulong b)
   if (!a) return primes_upto_zv(b);
   if (b < a) return cgetg(1, t_VECSMALL);
   d = b - a;
-  if (d > 100000UL) d = primepi_upper_bound(b) - primepi_lower_bound(a);
+  if (d > 100000UL)
+  {
+    ulong D = (ulong)ceil(primepi_upper_bound(b)-primepi_lower_bound(a));
+    if (D < d) d = D;
+  }
   return primes_interval_i(a, b, d);
 }
 GEN
