@@ -1,6 +1,4 @@
-/* $Id$
-
-Copyright (C) 2000  The PARI group.
+/* Copyright (C) 2000  The PARI group.
 
 This file is part of the PARI/GP package.
 
@@ -22,53 +20,50 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA. */
 static GEN
 dirzetak0(GEN nf, ulong N)
 {
-  GEN vect, c, c2, pol = nf_get_pol(nf), index = nf_get_index(nf);
-  pari_sp av = avma;
+  GEN vect, c, c2, T = nf_get_pol(nf), index = nf_get_index(nf);
+  pari_sp av = avma, av2;
   const ulong SQRTN = (ulong)(sqrt(N) + 1e-3);
   ulong i, p, lx;
-  byteptr d = diffptr;
   long court[] = {evaltyp(t_INT)|_evallg(3), evalsigne(1)|evallgefint(3),0};
+  forprime_t S;
 
   (void)evallg(N+1);
   c  = cgetalloc(t_VECSMALL, N+1);
   c2 = cgetalloc(t_VECSMALL, N+1);
   c2[1] = c[1] = 1; for (i=2; i<=N; i++) c[i] = 0;
-
-  maxprime_check(N);
-  for (p = 0; p <= N; avma = av)
+  u_forprime_init(&S, 2, N);
+  av2 = avma;
+  while ( (p = u_forprime_next(&S)) )
   {
-    NEXT_PRIME_VIADIFF(p, d);
-    court[2] = p;
-    if (umodiu(index, p)) /* court does not divide index */
+    avma = av2;
+    if (umodiu(index, p)) /* p does not divide index */
     {
-      vect = gel(FpX_degfact(pol,court),1);
+      vect = gel(Flx_degfact(ZX_to_Flx(T,p), p),1);
       lx = lg(vect);
     }
     else
     {
-      GEN P = idealprimedec(nf,court); lx = lg(P); vect = cgetg(lx,t_VECSMALL);
+      GEN P;
+      court[2] = p; P = idealprimedec(nf,court);
+      lx = lg(P); vect = cgetg(lx,t_VECSMALL);
       for (i=1; i<lx; i++) vect[i] = pr_get_f(gel(P,i));
     }
     if (p <= SQRTN)
       for (i=1; i<lx; i++)
       {
-        GEN NP = powiu(court, vect[i]); /* Norm P[i] */
-        ulong qn, q;
-
-        if (cmpiu(NP, N) > 0) break;
-        qn = q = (ulong)NP[2];
+        ulong qn, q = upowuu(p, vect[i]); /* Norm P[i] */
+        if (!q || q > N) break;
         memcpy(c2 + 2, c + 2, (N-1)*sizeof(long));
         /* c2[i] <- c[i] + sum_{k = 1}^{v_q(i)} c[i/q^k] for all i <= N */
-        while (qn <= (ulong)N)
+        for (qn = q; qn <= N; qn *= q)
         {
-          ulong k, k2; /* k2 = k*qn */
-          LOCAL_HIREMAINDER;
-          for (k = N/qn, k2 = k*qn; k > 0; k--, k2 -=qn) c2[k2] += c[k];
-          qn = mulll(qn, q); if (hiremainder) break;
+          ulong k0 = N/qn, k, k2; /* k2 = k*qn */
+          for (k = k0, k2 = k*qn; k > 0; k--, k2 -=qn) c2[k2] += c[k];
+          if (q > k0) break; /* <=> q*qn > N */
         }
         swap(c, c2);
       }
-    else /* p > sqrt(N): much simpler */
+    else /* p > sqrt(N): simpler */
       for (i=1; i<lx; i++)
       {
         ulong k, k2; /* k2 = k*p */
@@ -77,6 +72,7 @@ dirzetak0(GEN nf, ulong N)
         for (k = N/p, k2 = k*p; k > 0; k--, k2 -= p) c[k2] += c[k];
       }
   }
+  avma = av;
   pari_free(c2); return c;
 }
 
@@ -86,10 +82,10 @@ dirzetak(GEN nf, GEN b)
   GEN z, c;
   long n;
 
-  if (typ(b) != t_INT) pari_err(talker,"not an integer type in dirzetak");
+  if (typ(b) != t_INT) pari_err_TYPE("dirzetak",b);
   if (signe(b) <= 0) return cgetg(1,t_VEC);
   nf = checknf(nf);
-  n = itos_or_0(b); if (!n) pari_err(talker,"too many terms in dirzetak");
+  n = itou_or_0(b); if (!n) pari_err_OVERFLOW("dirzetak");
   c = dirzetak0(nf, n);
   z = vecsmall_to_vec(c); pari_free(c); return z;
 }
@@ -106,7 +102,7 @@ zeta_get_limx(long r1, long r2, long bit)
   c1 = mulrs(powrfrac(real2n(1, DEFAULTPREC), -2*r2, N), N);
 
   p1 = powru(Pi2n(1, DEFAULTPREC), r - 1);
-  p2 = mulir(powuu(N,r), p1); setexpo(p2, expo(p2)-r2);
+  p2 = mulir(powuu(N,r), p1); shiftr_inplace(p2, -r2);
   c0 = sqrtr( divrr(p2, powru(c1, r+1)) );
 
   A0 = logr_abs( gmul2n(c0, bit) ); p2 = divrr(A0, c1);
@@ -121,7 +117,7 @@ zeta_get_N0(GEN C,  GEN limx)
   pari_sp av = avma;
   GEN z = gcvtoi(gdiv(C, limx), &e); /* avoid truncation error */
   if (e >= 0 || is_bigint(z))
-    pari_err(talker, "need %Ps coefficients in initzeta: computation impossible", z);
+    pari_err_OVERFLOW("zeta_get_N0 [need too many primes]");
   if (DEBUGLEVEL>1) err_printf("\ninitzeta: N0 = %Ps\n", z);
   avma = av; return itos(z);
 }
@@ -178,7 +174,7 @@ initzeta(GEN T, long prec)
   GEN limx, resi,zet,C,coeflog,racpi,aij,tabj,colzero, tabcstn, tabcstni;
   GEN c_even, ck_even, c_odd, ck_odd, serie_even, serie_odd, serie_exp;
   GEN VOID;
-  long N0, i0, r1, r2, r, R, N, i, j, k, n, bit = bit_accuracy(prec) + 6;
+  long N0, i0, r1, r2, r, R, N, i, j, k, n, bit = prec2nbits(prec) + 6;
   pari_timer ti;
 
   pari_sp av, av2;
@@ -198,7 +194,7 @@ initzeta(GEN T, long prec)
     nf = bnf_get_nf(bnf);
   }
 
-  prec = (prec<<1) - 2;
+  prec = precdbl(prec);
   racpi = sqrtr(mppi(prec));
   /* all the above left on stack for efficiency */
 
@@ -248,7 +244,7 @@ initzeta(GEN T, long prec)
   for (k=1; k<R; k++)
   {
     GEN t = mulri(gel(zet,k), addis(shifti(gr2, k), r1));
-    setexpo(t, expo(t)-k);
+    shiftr_inplace(t, -k);
     if (k&1) togglesign(t);
     gel(ck_even,k) = t;
   }
@@ -256,7 +252,7 @@ initzeta(GEN T, long prec)
   for (k = 1; k <= r2+1; k++)
   {
     GEN t = mulri(gel(zet,k), subis(shifti(gru,k), r1));
-    setexpo(t, expo(t)-k);
+    shiftr_inplace(t, -k);
     if (k&1) togglesign(t);
     gel(ck_odd,k) = addsr(r, t);
   }
@@ -396,7 +392,7 @@ slambdak(GEN znf, long s, long flag, long prec)
   cstlog = gel(znf,6);
   coef   = gel(znf,8);
   coeflog= gel(znf,9);
-  i0 = lg(gel(C,1))-1;
+  i0 = nbrows(C);
   N0 = lg(coef)-1;
 
   if (s < 0 && (r2 || !odd(s))) s = 1 - s;
@@ -504,7 +500,7 @@ cxlambdak(GEN znf, GEN s, long flag, long prec)
   coeflog= gel(znf,9);
   r1 = mael(znf,1,1);
   r2 = mael(znf,1,2); r = r1+r2;
-  i0 = lg(gel(C,1))-1;
+  i0 = nbrows(C);
   N0 = lg(coef)-1;
   bigprec = precision(cst);
 
@@ -568,25 +564,25 @@ gzetakall(GEN znf, GEN s, long flag, long prec)
   pari_sp av = avma;
   GEN z;
 
-  if (typ(znf)!=t_VEC || lg(znf)!=10 || typ(znf[1]) != t_VECSMALL)
-    pari_err(talker,"not a zeta number field in zetakall");
+  if (typ(znf)!=t_VEC || lg(znf)!=10 || typ(gel(znf,1)) != t_VECSMALL)
+    pari_err_TYPE("zetakall", znf);
   if (isint(s, &s))
   {
     long ss = itos(s), r1, r2;
-    if (ss==1) pari_err(talker,"s = 1 is a pole (gzetakall)");
+    if (ss==1) pari_err_DOMAIN("zetak", "argument", "=", gen_1, s);
     znf_get_sign(znf, &r1, &r2);
     if (ss==0)
     {
       avma = av;
-      if (flag) pari_err(talker,"s = 0 is a pole (gzetakall)");
+      if (flag) pari_err_DOMAIN("zetak", "argument", "=", gen_0, s);
       if (r1 + r2 > 1) return gen_0;
       return r1? mkfrac(gen_m1, gen_2): gneg(gel(znf, 2));
     }
     if (!flag && ss < 0 && (r2 || !odd(ss))) return gen_0;
-    z = slambdak(znf, itos(s), flag, prec+1);
+    z = slambdak(znf, itos(s), flag, prec+EXTRAPRECWORD);
   }
   else
-    z = cxlambdak(znf, s, flag, prec+1);
+    z = cxlambdak(znf, s, flag, prec+EXTRAPRECWORD);
   if (gprecision(z) > prec) z = gprec_w(z, prec);
   return gerepileupto(av, z);
 }
